/*************************************************************************************** 

	PGLink2 firmware.  Kevin Mellott 2104.
	
	Updated for USB version.  Code improved after many years of practice.  
	Provides raw data print out to serial port at 115.2kbaud, 8,N,1, even parity.
	Also provides human readable PG data output.  Several commands available to switch
	operation or trigger functions.  See command print statement below or type "help"
	in terminal window to get more info.

	FW Versions 1.0, 1.1, and 1.2 work with atmega88pa.  FW Version 1.3 and higher
	only works with atmega328p due to increase memory size needed over atmega88pa. 

	Serial numbers also only started with atmega328p version.
	
	Based on original PGLinkbox code which was based on work of those 
	mentioned below.  

	************************************************************************************ 
	Originally "power.c" (c) manfredo 9/91 (manfredo@opal.cs.tu-berlin.de)
  	Developed on an ATARI 1040ST with TC 1.1 using a logic analyzer to get
  	the correct timings.

	ported to PC compatibles by
	Greg Alt 10/91
 
	galt@peruvian.utah.edu
	or galt@es.dsd.com

	Substantially rewritten by Dave Stampe (c) 1991: PWRFILT.C
  	No cash, no warranty, no flames.
  	This stuff works great, so gimme credit.
	dstamp@watserv1.uwaterloo.ca			17/10/91
	************************************************************************************ 

	Also uses information from these articles:
	"Reach Out and Touch Your Data", Byte Magazine
	"Interfacing the Power Glove to a Macintosh", by Marie Williams and Paul Green
	"HOME - brewed" by Zine. Issue One: June 7,1993 from 
	                www.textfiles.com_magazines_MISC_zine

    Also includes some button info from LoneKiltedNinja at forums.nesdev.com in thread
	"the Power Glove".  2009-2012 forum posts.

***************************************************************************************/

//#define DEBUG
//#define ECHO

#include "PGLink2.h"

//GLOBALS (I know, I know, bad programmer. It's just easier for simple code.
//*************************************************************************************** 
//RS232 REMOTE receive buffer globals
unsigned char RecBufREMOTE[remoteBufCount];
int remoteBufPos=0;
unsigned char remoteCurData=0;

//Glove data 
unsigned char gdata[12];//Only low level PG data fetch functions should access this var
glove_data   *glov;		//All other functions should use glove_data var
int ox = -1000;			/* last x,y for hysterisis */
int oy = -1000;

//Default Glove filter parameters. When written will save to EEPROM.
unsigned char XH=2; 	//XHYST
unsigned char YH=2;	 	//YHYST
unsigned char XA=8; 	//XACC
unsigned char YA=8; 	//YACC
unsigned char XX=2; 	//XXTEND
unsigned char YX=1; 	//YXTEND
int x1s = 0;			/* delayed 1 sample (for smoothed velocity test) */
int y1s = 0;
int x2s = 0;			/* delayed 2 samples */
int y2s = 0;
int lx = 0;				/* last good X,Y speed */
int ly = 0;
int lax = 0;			/* bad data "stretch" counter */
int lay = 0;
int lsx = 0;			/* X,Y "hold" values to replace bad data */
int lsy = 0;
int lcx = 0;			/* last X,Y speed for accel. calc. */
int lcy = 0;

//Flags for incoming command options
unsigned char contPG=0;
unsigned char flagHR=0;
unsigned char flagBIN=0;

//FW ver
unsigned char ver_major=1; //FW version
unsigned char ver_minor=4; //FW version
//SN and dev info.  This need to be set for each new unit.  Hard coded so end user can't
//easily change it.
char* dev="ATMEGA328P";
unsigned char ser_num=10;
unsigned char clock_byte=0x70;

//For printing strings. Running out of room so everything shares the same buffer for sprintf
char buffer [remoteBufCount];


//*************************************************************************************** 
// Interupt Service Routine for UART0 Receive byte
//
//*************************************************************************************** 
ISR(USART_RX_vect) 
{ 
   	// Code to be executed when the USART0 receives a byte here
   	//stuff UDR0 into receive buffer
   	if(remoteBufPos>=remoteBufCount){remoteBufPos=0;}
   	remoteCurData=RecBufREMOTE[remoteBufPos]=UDR0;
   	#ifdef ECHO
   		USART_Transmit(REMOTE,RecBufREMOTE[remoteBufPos]);
	#endif
   	remoteBufPos++;
}

//*************************************************************************************** 
// Main loop
//
//*************************************************************************************** 
int main(void){

	glov=(glove_data *)gdata; //create a convient glove container pointer to gdata

	_delay_ms(10);
	
	//Now setup the hardware and grab the power-up state
	setupHardware();
	
	sprintf(buffer,"\n\rPGLink2. 2014. v%d.%d\n\r", ver_major,ver_minor);
	USART_transmitSENT(REMOTE,buffer);

	while(1){
		//Process REMOTE Serial Port Messages
		//If curData is the '\r' Carriage return character then process possible cmd. 
		//Need if statements here to switch on strings.
		if(remoteCurData==0x0D){
			_disable_interrupt();
			
			if(SER){
				sprintf(buffer,"Device: %s. SN: %d.\n\r", dev,ser_num);
				USART_transmitSENT(REMOTE,buffer);
			}else if(VER){
				sprintf(buffer,"PGLink2. 2014. v%d.%d\n\r", ver_major,ver_minor);
				USART_transmitSENT(REMOTE,buffer);
			}else if(CONT){
				contPG=~contPG;
				USART_transmitSENT(REMOTE,"Continous mode toggled.\n\r");
			}else if(HI_RES){
				Hi_Res();
				USART_transmitSENT(REMOTE,"HI_RES Triggered.\n\r");
			}else if(RESET){
				//Reset flags so we're in a known state
				contPG=0;
				flagHR=0;
				flagBIN=0;
				//pg_reset();
				USART_transmitSENT(REMOTE,"RESET Triggered.\n\r");
			}else if(HELP){
				printMENU();
			}else if(BIN){
				flagBIN=~flagBIN;
				USART_transmitSENT(REMOTE,"Binary mode toggled.\n\r");
			}else if(XHYST){
				XH=(convertA2H(RecBufREMOTE[6]));
				sprintf(buffer,"XHYST: %d\n\r", XH);
				USART_transmitSENT(REMOTE,buffer);
				EEPROM_write(XH_EEPROM_ADDR,XH);
			}else if(YHYST){
				YH=(convertA2H(RecBufREMOTE[6]));
				sprintf(buffer,"YHYST: %d\n\r", YH);
				USART_transmitSENT(REMOTE,buffer);
				EEPROM_write(YH_EEPROM_ADDR,YH);
			}else if(XACC){
				XA=(convertA2H(RecBufREMOTE[5]));
				sprintf(buffer,"XACC: %d\n\r", XA);
				USART_transmitSENT(REMOTE,buffer);
				EEPROM_write(XA_EEPROM_ADDR,XA);
			}else if(YACC){
				YA=(convertA2H(RecBufREMOTE[5]));
				sprintf(buffer,"YACC: %d\n\r", YA);
				USART_transmitSENT(REMOTE,buffer);
				EEPROM_write(YA_EEPROM_ADDR,YA);
			}else if(XXTEND){
				XX=(convertA2H(RecBufREMOTE[7]));
				sprintf(buffer,"XXTEND: %d\n\r", XX);
				USART_transmitSENT(REMOTE,buffer);
				EEPROM_write(XX_EEPROM_ADDR,XX);
			}else if(YXTEND){
				YX=(convertA2H(RecBufREMOTE[7]));
				sprintf(buffer,"YXTEND: %d\n\r", YX);
				USART_transmitSENT(REMOTE,buffer);
				EEPROM_write(YX_EEPROM_ADDR,YX);
			}else if(STATE){
				sprintf(buffer,"\n\r********************PGLink2. 2014. v%d.%d********************\n\r",ver_major,ver_minor);
				USART_transmitSENT(REMOTE,buffer);
				sprintf(buffer,"Device: %s. SN: %d.\n\r", dev,ser_num);
				USART_transmitSENT(REMOTE,buffer);
				sprintf(buffer,"XHYST: %d\n\r", XH);
				USART_transmitSENT(REMOTE,buffer);
				sprintf(buffer,"YHYST: %d\n\r", YH);
				USART_transmitSENT(REMOTE,buffer);
				sprintf(buffer,"XACC: %d\n\r", XA);
				USART_transmitSENT(REMOTE,buffer);
				sprintf(buffer,"YACC: %d\n\r", YA);
				USART_transmitSENT(REMOTE,buffer);
				sprintf(buffer,"XXTEND: %d\n\r", XX);
				USART_transmitSENT(REMOTE,buffer);
				sprintf(buffer,"YXTEND: %d\n\r", YX);
				USART_transmitSENT(REMOTE,buffer);
			}

			//Command processed so setup for next time.
			_enable_interrupt();
			//Instead of flushing, which takes time, lets overwrite the first character
			RecBufREMOTE[0]=0;
			remoteBufPos=0;
			remoteCurData=0;
		}//end of if remoteCurData REMOTE serial processing end brace

		//Main glove data fetch section
		if(contPG==0){

			getGlove(); //fill glove array with data
			deglitch(glov);
			dehyst(glov);

		    if(flagBIN!=0){
				printPGBinary(glov);
			}else{
				printPG(glov);
				//Hi_Res(); //for debugging timing
			}	
		}
	}//end of main while loop
} //end of main

//*************************************************************************************** 
// void setupHardware(void)
//
// Setup Hardware here: UART buffer, UART, and PG port
//
//*************************************************************************************** 
void setupHardware(void){
	
	//Init RS232 buffer arrays
	flushBuf(RecBufREMOTE,remoteBufCount);

	//Setup calibrated oscillator byte.  See AVR053
	//You have to run this app note program to determine cal byte PER IC!!!
	//The note says it programs the byte but its not retained so you need to load it here.
	//I'm calibrating for clock frequency of 7.372800MHz @ 1% tolerance.  This clock freq
	//has 0% error for all baud rates for atmega88pa.
	OSCCAL=clock_byte; //0x66

	//Setup UART for Interupt driven mode
	setupUART();
	#ifdef DEBUG
		USART_transmitSENT(REMOTE,"USART Setup.\n\r");
	#endif

	//Read EEPROM glove filter values
	XH=EEPROM_read(XH_EEPROM_ADDR);
	YH=EEPROM_read(YH_EEPROM_ADDR);
	XA=EEPROM_read(XA_EEPROM_ADDR);
	YA=EEPROM_read(YA_EEPROM_ADDR);
	XX=EEPROM_read(XX_EEPROM_ADDR);
	YX=EEPROM_read(YX_EEPROM_ADDR);

	//Setup PG Glove pins
	PG_DIRPORT = (1<<PG_CLOCK)|(1<<PG_LATCH); //PG Clock and Latch as outputs
	PG_PORT	= 0; 	//set port to 0
	getGlove(); 	//Asking for data before issuing Hi Res seems to make it more 
					//reliable and repeatable at bootup
	_delay_ms(10);
	Hi_Res(); 		//Initialize PG to Hi Resolution mode or Absolute mode
	#ifdef DEBUG
		USART_transmitSENT(REMOTE,"PG Setup.\n\r");
	#endif

	//Enable Interrupts
	_enable_interrupt();
}

//*************************************************************************************** 
// void setupUART(void)
//
//*************************************************************************************** 
void setupUART(void){
	//USART Setup.  CHAN 0 is REMOTE, CHAN 1 is LCD; 
	USART_Init(3,REMOTE,1,2); 	//3-->115.2k 8,N,1 stop bit, 
								//Even parity

	//Enable interupt for UART0/UART1
	UCSR0B |= (1 << RXCIE0); //enable UART0 Receive interupt
}

//*************************************************************************************** 
// void flushBuf(volatile char *array, int len)
//
//*************************************************************************************** 
void flushBuf(volatile unsigned char *array, int len){
	for(int i=0; i<len ;i++){
		array[i]=0;
	}
}

//***************************************************************************************
// void Hi_Res(void)
//
// Powerglove Opcodes for Hi Res mode.
//
// Hi_Res timing is the Power Glove initialization.  This is implemented according to 
// PG Timing chart given on www.mellottsvrpage.com
//
// Some hard coded long delays in this function but they are only used in this function
// and are Hi_Res mode specific.
//
//*************************************************************************************** 
void Hi_Res(void)
{
 	int hi_res_codes[] = {0x06, 0xC1, 0x08, 0x00, 0x02, 0xFF, 0x01}; //code to set PG to HI_RES mode
	
	#ifdef DEBUG
		USART_transmitSENT(REMOTE,"Beginning Hi_Res.\n\r");
	#endif
  	//Beginning of sequence
	//Reset pulse which is latch L->H->L of 3us to 8us pulse width
 	pg_reset();

 	//read four bits and discard.  Some say it's not needed.
	_delay_us(D2BITS);
	PG_PORT = GNONE;
	_delay_us(D2BITS);
	PG_PORT = GCLOCK;
	_delay_us(D2WIDTH);
	PG_PORT = GNONE;
	_delay_us(D2BITS);
	PG_PORT = GCLOCK;
	_delay_us(D2WIDTH);
	PG_PORT = GNONE;
	_delay_us(D2BITS);
	PG_PORT = GCLOCK;
	_delay_us(D2WIDTH);
	PG_PORT = GNONE;
	_delay_us(D2BITS);
	PG_PORT = GCLOCK;

	//Handshake for commands
	PG_PORT = GCLOCK;
	_delay_ms(7.212); //DEFAULT of 7.212
	PG_PORT = GCLOLAT;
 	_delay_ms (2.260); //DEFAULT of 2.260.  This seems to be a very long reset pulse.
	_delay_us(D2LATCH);//May not be needed


	//Now we're going to send data
 	//There are 7 initialization bytes
	int x=0;
	for (int t=0;t<7;t++)
 	{ 
		x = hi_res_codes[t];
   		for (int j=0;j<8;j++)
   		{
   			
			if (x & 0x80) 
   			{	//Trigger latch with clock if MSB bit high
    			PG_PORT = GCLOLAT;
				_delay_us(DCLK2LAT); //need offset so latch goes high slightly before 
				PG_PORT = GLATCH;	 //clock goes low	
				_delay_us(D2BITS);
    			PG_PORT = GCLOLAT;
    		}else{ 
				//MSB is low so only need to trigger clock
				PG_PORT = GCLOCK;
				_delay_us(DCLK2LAT);  //to match timing of previous case
    			PG_PORT = GNONE;
				_delay_us(D2BITS);
    			PG_PORT = GCLOCK;
    		}
    		x =x<<1; //shift to the next bit
			_delay_us(D2WIDTH); //DEFAULT of D2WIDTH. 11 tuned to get 22us clock period
    	}
    	_delay_us (D2BYTES); //83
  	}

  	_delay_ms (0.892); //DEFAULT of 892.  800 tuned to achieve 892us delay
  	PG_PORT = GCLOCK;
  	_delay_ms (20);  //Default 40ms but 8MHz atmel has 32ms max. 
	_delay_ms (20);  //Delay may be longer due to other uC work but this guarantees 40ms.

	#ifdef DEBUG
		USART_transmitSENT(REMOTE,"End of Hi_Res.\n\r");
	#endif
}
//***************************************************************************************
// unsigned char getGlove(void)
//
// Main glove byte forming routine.
//
// We don't use all the bytes but there are 12 bytes of data so we'll read them all.
// 12 bytes is for high res mode.  Joystick mode only has 8 bytes. 
//
//*************************************************************************************** 
unsigned char getGlove(void)
{

 	for (int v=0;v<8;v++)
	{                               
 		gdata[v]=getByte();  //stores data into a array which can be accessed later.    
    	_delay_us (D2BYTES); //DEFAULT D2BYTES. 80 tuned
    }
	//gdata[v]=getByte();
	//_delay_ms(D2SLOW); //14.720. Doesn't seem to be required inbetween bytes 8 and 9

	for (int k=8;k<12;k++)
	{                               
 		gdata[k]=getByte();  //stores data into a array which can be accessed later.    
    	_delay_us (D2BYTES); //
    }
	_delay_ms(D2BUFFER); //Required if delay after getGlove is too short
	_delay_ms(D2BUFFER); 
	
	return 0;
}

//***************************************************************************************
// unsigned char getByte(void)
//
// Main glove bit reading sequence.  Forms a byte to be returned to getGlove()
//
//*************************************************************************************** 
unsigned char getByte(void)
{
	unsigned char gread=0;

	//Generate reset pulse
 	pg_reset();
	_delay_us(D2BITS); 

	for(int k = 0; k < 8; k++)  
 	{
 		gread<<=1;
		gread+=(PG_READ & (1<<PINC2))>>PINC2;
 		PG_PORT = GNONE;
 		_delay_us(D2BITS);   
 		PG_PORT = GCLOCK;
 		_delay_us(D2WIDTH); 
 	}  
	    
 	return gread;
} 

//***************************************************************************************
// unsigned char pg_reset(void)
//
// Reset pulse which is Latch L->H->L
// This tells the PG to output bit 0 of byte.  Subsequent data is output via clock pulses
// going low.
//
//*************************************************************************************** 
unsigned char pg_reset(void){
	
	//PG_PORT = GNONE;
	//_delay_us(D2LATCH);
	PG_PORT = GCLOCK;
	//_delay_us(D2LATCH);
	PG_PORT = GCLOLAT;
	_delay_us(D2LATCH);
 	PG_PORT = GCLOCK;

	return 0;
}

//***************************************************************************************
// dehyst(glove_data *g)
//
// Copied and pasted from PGlove.zip, THEGLOVE.C file by Greg Alt, Oct. 1991.
//
// Would be interesting to implement my own filtering functions but we'll start with these.
//
//*************************************************************************************** 
void dehyst(glove_data *g)		/* hysterisis deglitch (low noise removal) */
{
	
	int x = g->x;
	int y = g->y;

 	if(g->keys==0) ox = oy = 0;	/* handle recentering ("0"key or "Center") */

 	if(x-ox>XH) ox = x-XH;		/* X hysterisis */
 	if(ox-x>XH) ox = x+XH;

 	if(y-oy>YH) oy = y-YH;		/* Y hysterisis */
 	if(oy-y>YH) oy = y+YH;

 	g->x = ox;					/* replace present X,Y data */
 	g->y = oy;
}

//***************************************************************************************
// void deglitch(glove_data *g)
//
// Copied and pasted from PGlove.zip, THEGLOVE.C file by Greg Alt, Oct. 1991.
//
// Would be interesting to implement my own filtering functions but we'll start with these.
//
//*************************************************************************************** 
void deglitch(glove_data *g)
{
 	int vx, vy;

 	int x = g->x;
 	int y = g->y;

 	if(g->keys==0)		/* reset on recentering ("0" or "Center" key) */
  	{
   		x1s = x2s = y1s = y2s = 0;
   		lx = ly = lax = lay = 0;
   		lsx = lsy = lcx = lcy = 0;
  	}

 	vx = x-((x1s+x2s)>>1);		/* smoothed velocity */
 	vy = y-((y1s+y2s)>>1);

 	x2s = x1s;			/* update last values */
 	x1s = g->x;

 	y2s = y1s;
 	y1s = g->y;

 	if(abs(lcx-vx)>XA) lax = XX;	/* check for extreme acceleration */
 	if (lax == 0) lx=vx;                   /* save only good velocity        */
 	lcx = vx;                              /* save velocity for next accel.  */

 	if(abs(lcy-vy)>YA) lay = YX;	/* same deal for Y accel. */
 	if (lay == 0) ly=vy;
 	lcy = vy;

 	if(lax!=0)		/* hold X pos'n if glitch */
  	{
   		g->x = lsx;
   		lax--;
  	}

 	if(lay!=0)             /* hold Y pos'n if glitch */
  	{
   		lay--;
   		g->y = lsy;
  	}

 	lsx = g->x;		/* save position for X,Y hold */
 	lsy = g->y;

	/* g->y = x;*/
}

//***************************************************************************************
// void printMENU(void)
//
// HELP menu print function 
//
// 8/17/14. Right now adding the additional print statements crashes the atmega88pa.  
//			Something with code size because changing it crashes everything else even when 
//			I don't call this function. Use an atmega168pa next time??
//
//*************************************************************************************** 
void printMENU(void){

	sprintf(buffer,"\n\r********************PGLink2. 2014. v%d.%d********************\n\r",ver_major,ver_minor);
	USART_transmitSENT(REMOTE,buffer);
	USART_transmitSENT(REMOTE,"SER       --> Returns Serial Number.\n\r");
	USART_transmitSENT(REMOTE,"VER       --> Returns FW Version.\n\r");
	USART_transmitSENT(REMOTE,"STATE     --> Prints Sn, FW Ver., and filter settings.\n\r");
	USART_transmitSENT(REMOTE,"HELP      --> Prints Commands.\n\r");
	USART_transmitSENT(REMOTE,"CONT      --> Toggles PG streaming.\n\r");
	USART_transmitSENT(REMOTE,"HI_RES    --> Issues Hi_Res Init.\n\r");
	USART_transmitSENT(REMOTE,"RESET     --> Issues Reset command.\n\r");
	USART_transmitSENT(REMOTE,"BIN       --> Toggles binary PG print.\n\r");
	USART_transmitSENT(REMOTE,"XHYST  a  --> Sets X hysteresis filter value.\n\r");
	USART_transmitSENT(REMOTE,"YHYST  a  --> Sets Y hysteresis filter value.\n\r");
	USART_transmitSENT(REMOTE,"XACC   a  --> Sets X acceleration filter value.\n\r");					
	USART_transmitSENT(REMOTE,"YACC   a  --> Sets Y acceleration filter value.\n\r");	
	USART_transmitSENT(REMOTE,"XXTEND a  --> Sets X deglitching time.\n\r");	
	USART_transmitSENT(REMOTE,"YXTEND a  --> Sets Y deglitching time.\n\r\n\r");
}

//***************************************************************************************
// unsigned char printPG(unsigned char *data)
//
// Prints the data glove data to a human readable form per below.
//
// The glove_data array has been simplified. These are its functions:
//
// Byte 0: header byte of $A0
//
// Byte 1,2,3: 
//		x =   X position, 3mm per number. 2's complement. -128 to 127
//		y =   Y position, 3mm per number. 2's complement. -128 to 127
//		z =   distance,  14mm per number. 2's complement. -128 to 127
//
// Byte 4:
//		rot = wrist twist. 0 is up 1 is slightly CW, 5 is down, 0hB is slightly CCW.
//      About 30 to 40 degrees per count.
//
// 		Note: exact scaling of all above change with distance! Closer is higher res.
//
// Byte 5:
//		fingers = packed 2-bit values, 0 is open, 3 is (tight) fist:
//	  	Bit format: TtIiMmRr  for Thumb, Index, Middle, and Ring fingers.
//
// Byte 6:
//		keys: $FF is no key. Responds with 0 to 9 for keys "0" thru "9"
//      $82 = START, $83 = SEL, $0A = "A", $0B = "B", 0 is "Center"
//      Up,down,left,right are $0D,$0E,$0C,$0F respectively.
//      $81 is PROG and $80 might be ENTER or no key.
//
// Byte 7,8: Always 0
//
// Byte 9 is occlusion of mics and speakers.
//		Value: 000abcdef in binary.  abc: right TRX, def: left TRX.
//		a/d lower-right mic; b/e upper right mic; cf upper-left mic
//		Bits are normally 1 but will drop to 0 if speaker-mic data lost.
//		e.g. if bit 0(f) drops to 0, the left TRX isn't detected by the upper-left mic
//
// Byte 10,11: Always $FF
//
//***************************************************************************************
unsigned char printPG(glove_data *g){
	
	char *keys=" ";

	//Now print data

	//Header "A0" and x,y,z data
	//Only print if PG data is valid
	if((g->header)==0xA0){
		sprintf(buffer,"X: %+.3hhd, Y: %+.3hhd, Z: %+.3hhd, ",(signed char)(g->x),(signed char)(g->y),(signed char)(g->z));
		USART_transmitSENT(REMOTE,buffer);

		//rotation and finger data
		sprintf(buffer,"R: %.3d, F: %d%d%d%d, ",g->rot,(0xC0 & (g->fingers))>>6,(0x30 & (g->fingers))>>4,(0x0C & (g->fingers))>>2,(0x03 & (g->fingers)));
		USART_transmitSENT(REMOTE,buffer);

		//Key data.  
		switch(g->keys){
			case 0: keys="0"; //shared with Center button in Hi_Res mode.
			break;
			case 1: keys="1";
			break;
			case 2: keys="2";
			break;
			case 3: keys="3";
			break;
			case 4: keys="4";
			break;
			case 5: keys="5";
			break;
			case 6: keys="6";
			break;
			case 7: keys="7";
			break;
			case 8: keys="8";
			break;
			case 9: keys="9";
			break;
			case 0x80: keys="ENT";
			break;
			case 0x81: keys="PRO";
			break;
			case 0x82: keys="STA";
			break;
			case 0x83: keys="SEL";
			break;
			case 0x0A: keys="AAA";
			break;
			case 0x0B: keys="BBB";
			break;
			case 0x0D: keys=" UP";
			break;
			case 0x0E: keys="DWN";
			break;
			case 0x0C: keys="LFT";
			break;
			case 0x0F: keys="RGT";
			break;

			default: keys="N/A"; //N for NONE
			break;
		}
		//Print keys and occlusion byte
		sprintf(buffer,"K: %s, O: %2x\n\r",keys,(unsigned char)(g->occ));
		USART_transmitSENT(REMOTE,buffer);

		//Bytes 7,8 are always zero
		//Byte 9 is occulusion byte printed above
		//Bytes 10, 11 are always FF.
	}

	return 0;
}

//***************************************************************************************
// unsigned char printPGBinary(unsigned char *data)
//
// Prints raw hex binary data from PG glove
//
//***************************************************************************************
unsigned char printPGBinary(glove_data *g){

	unsigned char* data=(unsigned char*)g;

	for(int j=0; j<12; j++){
		//sprintf(buffer,"%x",data[j]);
		//USART_transmitSENT(REMOTE,buffer);
		USART_Transmit(data[j],REMOTE);
	}

	_delay_ms(5.4); //binary is too fast for the PG. 

	return 0;
}
