//********************************************************************************************
//UARTDriver header file
//
//********************************************************************************************

#include "UARTDriver_644.h"

/****************************USART Functions *****************************************/
//************************************************************************************
// InitUART function definition
//
// Precondition: baudrate function has been set.
// 
// Postcondition: Writes the chosen baudrate to the appropriate register.  Initializes the UART
//                transmitter and receiver. 
//				  parity is 0-disabled, 1-reserved, 2-even, 3-odd 
//				  stopBits is 0-1 bit 1- 2 bits
// 
//*************************************************************************************
void USART_Init( unsigned int ubrr, unsigned char chan, 
					unsigned char stopBits, unsigned char parity)
{	
	if(chan==0){
		/* Set baud rate */
		UBRR0H = (unsigned char)(ubrr>>8);
		UBRR0L = (unsigned char)ubrr;
		/* Enable receiver and transmitter */
		UCSR0B = (1<<RXEN0)|(1<<TXEN0);
		//UCSR0A = (1<<U2X0);
		/* Set frame format: 8 data bits, 2stop bit, Even parity (UPM1=1 and UPM0=0) */
		UCSR0C = ((stopBits-1)<<USBS0)|(parity<<UPM00)|(3<<UCSZ00); //for atmega644
	}else if(chan==1){
		/* Set baud rate */
		UBRR1H = (unsigned char)(ubrr>>8);
		UBRR1L = (unsigned char)ubrr;
		/* Enable receiver and transmitter */
		UCSR1B = (1<<RXEN1)|(1<<TXEN1);
		//UCSR1A = (1<<U2X1);
		/* Set frame format: 8 data bits, 2stop bit, Even parity (UPM1=1 and UPM0=0) */
		UCSR1C = ((stopBits-1)<<USBS1)|(parity<<UPM10)|(3<<UCSZ10); //for atmega644
	}		
}
 

//*************************************************************************************
// TransmitByte function definition
//
// Precondition: data variable is set.
// 
// Postcondition: Waits until the UART is ready to receive vaild data and sets the data 
//                register to the desired value.  
// 
//*************************************************************************************
void USART_Transmit(unsigned char data, unsigned char chan)
{	
	if(chan==0){
		/* Wait for empty transmit buffer */
		while ( !( UCSR0A & (1<<UDRE0)) )
		;
		/* Put data into buffer, sends the data */
		UDR0 = data;
	}else if(chan==1){
		/* Wait for empty transmit buffer */
		while ( !( UCSR1A & (1<<UDRE1)) )
		;
		/* Put data into buffer, sends the data */
		UDR1 = data;	
	}
}


//*************************************************************************************
// TransmitByte 16bit function definition
//
// Precondition: data variable is set.
// 
// Postcondition: Handles the shifting of a 16bit integer into two 8bit bytes so we can
//				  send it out the 8bit USART.  
// 
//*************************************************************************************
 
void USART_Transmit_16bit(unsigned short data, unsigned char chan)
{
	unsigned char lowByte=0;
	unsigned char highByte=0;
	
	//recover high and low byte from axis measurement
	//so we can send out 8 bit USART
	//read the 1st byte
	lowByte=(unsigned char)data;
	//shift the byte to the right 8 bits 
	data >>= 8;
	//read the high byte
	highByte=(unsigned char)data;
	
	if(chan==0){
		USART_Transmit(highByte,0);
		USART_Transmit(lowByte,0);
	}else if(chan==1){
		USART_Transmit(highByte,1);
		USART_Transmit(lowByte,1);
	}
}


//**************************************************************************************
// ReceiveData function definition
//
// Precondition: None
// 
// Postcondition: Waits until the UART has received vaild data and returns the value 
//					of the data register.
// 
//************************************************************************************** 
unsigned char USART_Receive(unsigned char chan)
{
	if(chan==0){
		/* Wait for data to be received */
		while ( !(UCSR0A & (1<<RXC0)) )
		;
		/* Get and return received data from buffer */
		return UDR0;
	}else if (chan==1){
		/* Wait for data to be received */
		while ( !(UCSR1A & (1<<RXC1)) )
		;
		/* Get and return received data from buffer */
		return UDR1;
	}
	return 0;
}


//***************************************************************************************
// UART flush function definition
//
// Precondition: None
// 
// Postcondition: Receive buffer has been flushed.
// 
//***************************************************************************************  

void USART_Flush(unsigned char chan)
{
	unsigned char dummy;
	if(chan==0){
		while ( UCSR0A & (1<<RXC0) ) dummy = UDR0;
	}else if (chan==1){
		while ( UCSR1A & (1<<RXC1) ) dummy = UDR1;
	}
}


//***************************************************************************************
// UART bootup message
//
// Precondition: UART setup and initialization function called.
// 
// Postcondition: String printed to screen.
// 
//*************************************************************************************** 
void USART_bootup(unsigned char chan){
	char bootup0[] = "UART0 Initializing...\n\r";
	char bootup1[] = "UART1 Initializing...\n\r";

	for(int i=0; i<strlen(bootup0); i++){
		if(chan==0){
			USART_Transmit(bootup0[i],0);
		}else if (chan==1){
			USART_Transmit(bootup1[i],1);
		}
	}
}

//***************************************************************************************
// UART Transmit Sentence/String
//
// Precondition: UART setup and initialization function called.
// 
// Postcondition: String printed to screen.
// 
//*************************************************************************************** 
void USART_transmitSENT(unsigned char chan, char * buf){
	for(int i=0; i<strlen(buf); i++){
		if(chan==0){
			USART_Transmit(buf[i],0);
		}else if (chan==1){
			USART_Transmit(buf[i],1);
		}
	}
}


//UART Write for VFX1_Linkbox.cpp.  Outputs {sourceAddr,msglen,msg,checksum} to COM port
//Need this for easier software interfacing
void UART_Transmit_GlobalArray(unsigned char source, unsigned char msgSize, char* msg){
	unsigned char checkSumGlobal;
	char globalArray[50];	
	
	globalArray[0]=source; //address for VFX1 Linkbox
	globalArray[1]=msgSize;  //msgSize
	checkSumGlobal=globalArray[0]^globalArray[1];
	sprintf(globalArray+2,"%s",msg);
	//calculate checksum
	for(unsigned char i=0; i<(msgSize); i++){
		checkSumGlobal^=msg[i];
	}
	globalArray[msgSize+2]=checkSumGlobal;
	globalArray[msgSize+3]='\0';  //overwrote NULL byte with checkSum so add it to the end
	USART_transmitSENT(0,globalArray);
}


//*************************************************************************************** 
//Interupt Service Routine for UART Receive byte
//*************************************************************************************** 
/*ISR(USART0_RX_vect) 
{ 
   // Code to be executed when the USART0 receives a byte here
   //stuff UDR0 into receive buffer
   for(int i=5; i >0; i--){
    	       RecBuf[i] = RecBuf[i-1];
        }
   RecBuf[0] = UDR0;
}*/

